@@ -25,32 +25,33 @@ import shutil
 import os
 import re
 
-from rvbd.common.exceptions import *
+from rvbd.common.exceptions import RvbdException, RvbdHTTPException
 from rvbd.common.utils import DictObject
 from rvbd.common.http_pool import ConnectionPool
 
-__all__ = [ 'Connection' ]
+__all__ = ['Connection']
 
 logger = logging.getLogger(__name__)
 
+
 class Connection(object):
-    '''This class is handles all HTTP/HTTPS communication with a device,
+    """This class handles all HTTP/HTTPS communication with a device,
     including connection management, connection pooling, XML and JSON
     requests and responses.  This class is normally instantiated
-    by a `Service` object, but may be separately as needed.
+    by a `Service` object, but may be used separately as needed.
 
     Authentication is not handled directly by this class.  The caller
     must set the appropriate headers for authentication either
     using the `extra_headers` parameter or by calling the `add_headers`
     method.  The latter will affect all subsequent requests.
-    '''
+    """
 
     HTTPLIB_DEBUGLEVEL = 0
     DEBUG_MSG_BODY = 0
 
     def __init__(self, hostname, port=None, force_ssl=True, pool_size=1,
                  reauthenticate_handler=None, test_resource=None):
-        '''Create a connection to the requested host.
+        """Create a connection to the requested host.
 
         `hostname` is the name or IP address of the device to connect to
 
@@ -68,15 +69,15 @@ class Connection(object):
         `reauthenticate_handler`, if set, should contain callable that is
          invoked if a request returns a session expiration error.
          If the call succeeds, then the request is retried once.
-        '''
+        """
 
         self.hostname = hostname
         if port is None:
-            self.ports = [ 443, 80 ]
+            self.ports = [443, 80]
         elif isinstance(port, list):
             self.ports = port
         else:
-            self.ports = [ port ]
+            self.ports = [port]
         self.force_ssl = force_ssl
 
         self.conn = None
@@ -113,7 +114,8 @@ class Connection(object):
             self.conn.connect()
 
     def _tryconnect(self, port):
-        '''Try connecting to the given port, both with and without SSL.
+        """Try connecting to the given port, both with and without SSL.
+
         Upon success, set self.port and self.ssl and return True.
 
         If a connection cannot be established, return False.
@@ -121,7 +123,7 @@ class Connection(object):
         to the caller.
 
         Note that this method does not authenticate to the appliance.
-        '''
+        """
 
         # this helper function does all the work of trying to connect
         # and determine the protocol information on the given port
@@ -132,15 +134,16 @@ class Connection(object):
         # below can simply make a guess about the right ssl setting and
         # simply call this function again if it guesses wrong.
         #
-        # it also handles
-        class SSLMismatch(Exception): pass
+        class SSLMismatch(Exception):
+            pass
+
         def attempt(port, ssl):
             try:
                 self._reset_connection(self.hostname, port, ssl)
                 # Test the connection with *any* resource -- it need not be
                 # a valid resource, just something that will verify connectivity
                 if self.test_resource:
-                    rsc = "/foobar" if self.test_resource == True else self.test_resource
+                    rsc = "/foobar" if self.test_resource is True else self.test_resource
                     self.conn.request("GET", rsc)
                     resp = self.conn.getresponse()
 
@@ -215,7 +218,7 @@ class Connection(object):
                 return False
 
     def _request(self, urlpath, method, body, headers):
-        '''Internal request method that will retry on failure as directed.'''
+        """Internal request method that will retry on failure as directed."""
         retry = True
         while True:
             try:
@@ -225,12 +228,14 @@ class Connection(object):
                 resp = self.conn.getresponse()
                 return resp
                 
-            except (httplib.ResponseNotReady), e:
+            except httplib.ResponseNotReady, e:
                 # User probably forgot to drain the response
-                raise RvbdException('Cannot send request, exception httplib.ResponseNotReady - usually means response from last request was not drained with read()')
+                raise RvbdException('Cannot send request, exception httplib.ResponseNotReady - '
+                                    'usually means response from last request was not drained with read()')
                 
-            except (httplib.CannotSendRequest), e:
-                if (not retry) or (self._pool_size != 1): raise e
+            except httplib.CannotSendRequest, e:
+                if not retry or self._pool_size != 1:
+                    raise e
             
             except httplib.BadStatusLine, e:
                 # stupid httplib is supposed to put the status line in e.line
@@ -249,10 +254,9 @@ class Connection(object):
             logger.info('Resetting connection')
             self._reset_connection(self.hostname, self.port, self.ssl)
                     
-            
     def request(self, urlpath, method='GET', body='', params=None,
                 extra_headers=None):
-        '''Send a generic HTTP request to the host.  
+        """Send a generic HTTP request to the host.  
 
         On success, the httplib response object is returned.  Any
         pending data must be read by the caller.
@@ -263,7 +267,7 @@ class Connection(object):
         `urlpath` is the full path of the resource from the initial slash.
             For example "/api/common/1.0/ping"
             
-        `method` is the HTPP method to use.  Default is GET.
+        `method` is the HTTP method to use.  Default is GET.
 
         `body` is the full text to send as the body of the message as is.
 
@@ -272,12 +276,12 @@ class Connection(object):
 
         `extra_headers` is a dictionary of additional HTTP headers to send
             with the request.
-        '''
+        """
 
         if params:
             urlpath = "%s?%s" % (urlpath, urllib.urlencode(params))
 
-        logger.debug("http %s on url %s" % (method , urlpath))
+        logger.debug("http %s on url %s" % (method, urlpath))
 
         # XXXCJ - need to "lowercase" all headers to make sure there are
         # no duplicates due to changed capitalization
@@ -287,10 +291,10 @@ class Connection(object):
             headers.update(extra_headers)
 
         #fix for bug http://bugs.python.org/issue14721
-        if (body=='' or body is None):
+        if body == '' or body is None:
             if 'content-length' not in headers:
                 headers['content-length'] = "0"
-        elif (method == 'GET'):
+        elif method == 'GET':
             raise ValueError("data cannot be included for %s requests" % method)
 
         # XXXWP - We can't set  the  content  length  calling  len(body)  since
@@ -305,7 +309,7 @@ class Connection(object):
             if len(body) > self.DEBUG_MSG_BODY:
                 trunc = ' (truncated)'
 
-            logger.debug('Request body:\n%s%s\n' % \
+            logger.debug('Request body:\n%s%s\n' %
                          (body[:self.DEBUG_MSG_BODY], trunc))
 
         self.last_http_response = None
@@ -325,8 +329,8 @@ class Connection(object):
             # to reauthenticate, try to do so. Temporarily clear the
             # handler to avoid infinite recursion or multiple attempts
             # to reauthenticate.
-            if self._reauthenticate_handler is not None and \
-                   exc.error_id in ('AUTH_INVALID_SESSION', 'AUTH_EXPIRED_TOKEN'):
+            if (self._reauthenticate_handler is not None and
+                exc.error_id in ('AUTH_INVALID_SESSION', 'AUTH_EXPIRED_TOKEN')):
 
                 logger.debug('session timed out -- reauthenticating')
                 handler = self._reauthenticate_handler
@@ -352,11 +356,13 @@ class Connection(object):
 
     def xml_request(self, urlpath, method='GET', body='', params=None,
                     extra_headers=None):
-        '''Send an XML request to the host.  This is similar to `request`
-        except the Content-Type and Accept headres are set to text/xml.
-        In addition, any response will be XML-decoded as an
-        xml.etree.ElementTree.  The body is assumed to be an XML encoded
-        text string and is inserted into the HTTP payload as-is.'''
+        """Send an XML request to the host.  
+        
+        This is similar to `request` except the Content-Type and Accept headres
+        are set to text/xml.  In addition, any response will be XML-decoded as
+        an xml.etree.ElementTree.  The body is assumed to be an XML encoded
+        text string and is inserted into the HTTP payload as-is.
+        """
 
         # XXX/demmer for consistency this should handle the case where
         # body is an ElementTree.Element
@@ -393,10 +399,12 @@ class Connection(object):
 
     def json_request(self, urlpath, method='GET', data=None, params=None,
                      extra_headers=None):
-        '''Send a JSON rquest to the host.  For POST/PUT requests, the
-        `data` parameter will be JSON-encoded before transmission.  For
-        all requests, the response is JSON-decoded using the
-        DictObject class before returning.'''
+        """Send a JSON request to the host.
+        
+        For POST/PUT requests, the `data` parameter will be JSON-encoded before
+        transmission.  For all requests, the response is JSON-decoded using the
+        DictObject class before returning.
+        """
 
         if data is not None:
             body = json.dumps(data, cls=self.JsonEncoder)
@@ -419,7 +427,7 @@ class Connection(object):
             if len(data) > self.DEBUG_MSG_BODY:
                 trunc = ' (truncated)'
 
-            logger.debug('Response body:\n%s%s\n' % \
+            logger.debug('Response body:\n%s%s\n' %
                          (data[:self.DEBUG_MSG_BODY], trunc))
 
         # In some cases the device may return a 200 with no actual data
@@ -427,7 +435,7 @@ class Connection(object):
         # the client behavior (e.g. Rvbd-Shark-Status), so check for
         # both status code of 204 or an empty data response
         if resp.status == 204 or len(data) == 0:
-            return # no data
+            return  # no data
 
         t = resp.getheader('Content-type')
         if t != 'application/json':
@@ -435,10 +443,30 @@ class Connection(object):
 
         return json.loads(data, object_hook=DictObject.create_from_dict)
 
+    def upload_file(self, urlpath, data, params=None, extra_headers=None):
+        '''Send a JSON rquest to the host.  For POST/PUT requests, the
+        `data` parameter will be JSON-encoded before transmission.  For
+        all requests, the response is JSON-decoded using the
+        DictObject class before returning.'''
+
+        if extra_headers is None:
+            extra_headers = {}
+
+        extra_headers['Content-Type'] = 'application/octet-stream'
+
+        resp = self.request(urlpath, "PUT", data,
+                            params=params, extra_headers=extra_headers)
+
+        data = resp.read()
+        if resp.status == 204:
+            return # no data
+        elif resp.status == 201:
+            # created resource
+            return {'Location-Header': resp.getheader('location', '')}
+        return data
+
     def post_raw(self, urlpath, data, params=None, extra_headers=None):
-        '''
-        Upload raw data to the given URL path with the given
-        content type.
+        """Upload raw data to the given URL path with the given content type.
 
         `data` may be either a string or a python file object.
 
@@ -450,27 +478,27 @@ class Connection(object):
 
         Returns location information if resource has been created,
         otherwise the response body (if any).
-        '''
+        """
         resp = self.request(urlpath, "POST", data, params=params,
                             extra_headers=extra_headers)
         data = resp.read()
         if resp.status == 204:
-            return # no data
+            return  # no data
         elif resp.status == 201:
             # created resource
             return {'Location-Header': resp.getheader('location', '')}
         return data
 
-    def get_raw(self, urlpath, local_path, local_file_name = None,
-                local_file_overwrite = False, extra_headers=None, params=None):
-        '''Download a file from a remote URI and save it to a local path'''
+    def get_raw(self, urlpath, local_path, local_file_name=None,
+                local_file_overwrite=False, extra_headers=None, params=None):
+        """Download a file from a remote URI and save it to a local path"""
 
         # Get request
         resp = self.request(urlpath, method='GET', params=params, extra_headers=extra_headers)
 
         # Check if the user specified a file name
-        if local_file_name == None:
-            # Retrive the file name form the HTTP header
+        if local_file_name is None:
+            # Retrieve the file name form the HTTP header
             local_file_name = resp.getheader('Content-Disposition').split('=')[1]
 
         # Get the Full path
@@ -486,11 +514,11 @@ class Connection(object):
             shutil.copyfileobj(resp, fd)
 
     def add_headers(self, headers):
-        '''Add the dictionary HEADERS to the list of customer headers for all requests'''
+        """Add the dictionary HEADERS to the list of customer headers for all requests"""
         self._headers.update(headers)
 
     def del_headers(self, headers):
-        '''Drop any custom headers in the list of header names in HEADERS'''
+        """Drop any custom headers in the list of header names in HEADERS"""
         for h in headers:
             if h in self._headers:
                 del self._headers[h]
